//////////////////////////////////////////////////////////////////////////////
//
//    SCREENSHOOTDIALOG.CPP
//
//    Copyright (C) 2000-2003 Daniel M.Duley <dan.duley@verizon.net>
//                                              (from Pixie project)
//    Copyright (C) 2004 Gilles CAULIER <caulier.gilles@free.fr>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////

#include "acquireimagedialog.h"

#include <qlayout.h>
#include <qlabel.h>
#include <qtimer.h>
#include <qpushbutton.h>
#include <qfileinfo.h>
#include <qapplication.h>
#include <qwhatsthis.h>
#include <qcheckbox.h>

#include <kapplication.h>
#include <klocale.h>
#include <knotifyclient.h>
#include <kdebug.h>
#include <kconfig.h>
#include <kmessagebox.h>
#include <knuminput.h>

#include <unistd.h>

#include "screenshotdialog.h"


//////////////////////////////////// CONSTRUCTOR ////////////////////////////////////////////

ScreenGrabDialog::ScreenGrabDialog(QWidget *parent, const char *name)
                 :KDialogBase(parent, name, false, i18n("Digikam Screenshot Images Plugin"),
                 Help|User1|Close|User2, Close, true, i18n("&About"), i18n("&New snapshot"))
{
    setHelp("plugin-screenshotimages.anchor", "digikam");
    m_inSelect = false;
    QWidget* box = new QWidget( this );
    setMainWidget(box);
    QVBoxLayout *layout = new QVBoxLayout(box, 4);

    //---------------------------------------------

    QLabel *label1 = new QLabel(i18n("This dialog will grab either your desktop or a single\n"
                                     "application window. If you grab a single window your mouse\n"
                                     "cursor will change into crosshairs and simply select the\n"
                                     "window with your mouse."), box);
    layout->addWidget(label1);

    //---------------------------------------------

    m_desktopCB = new QCheckBox(i18n("Grab the entire desktop."), box);
    QWhatsThis::add( m_desktopCB, i18n( "<p>If you enable this option, the entire desktop will be grabbed, "
                                        "else only the active windows." ) );
    layout->addWidget(m_desktopCB);

    //---------------------------------------------

    m_hideCB = new QCheckBox(i18n("Hide all Digikam windows."), box);
    QWhatsThis::add( m_hideCB, i18n( "<p>If you enable this option, all Digikam windows will be hidden, "
                                     "during the grab operation." ) );
    layout->addWidget(m_hideCB);

    //---------------------------------------------

    QLabel *label2 = new QLabel(i18n("Delay:"), box);
    layout->addWidget(label2);
    m_delay = new KIntNumInput(box);
    QWhatsThis::add( m_delay, i18n( "<p>The delay in seconds before to start the grab operation.") );
    m_delay->setRange(1, 60);
    layout->addWidget(m_delay);
    layout->addStretch(1);

    //---------------------------------------------

    connect(this, SIGNAL(user1Clicked()), this, SLOT(slotAbout()));
    connect(this, SIGNAL(user2Clicked()), this, SLOT(slotCheckIfDelay()));
    connect(this, SIGNAL(closeClicked()), this, SLOT(slotClose()));

    //---------------------------------------------

    // Read all settings from configuration file.

    m_config = new KConfig("digikamrc");
    m_config->setGroup("ScreenshotImages Settings");

    if (m_config->readEntry("GrabDesktop", "true") == "true")
        m_desktopCB->setChecked( true );
    else
        m_desktopCB->setChecked( false );

    if (m_config->readEntry("HideDigikam", "true") == "true")
        m_hideCB->setChecked( true );
    else
        m_hideCB->setChecked( false );

    m_delay->setValue(m_config->readNumEntry("Delay", 1));

    delete m_config;
};


//////////////////////////////////// DESTRUCTOR /////////////////////////////////////////////

ScreenGrabDialog::~ScreenGrabDialog()
{
}


//////////////////////////////////////// SLOTS //////////////////////////////////////////////

void ScreenGrabDialog::slotAbout( void )
{
    KMessageBox::about(this, i18n("A Digikam plugin for grab images from screen\n\n"
                                  "Author: Gilles Caulier\n\n"
                                  "Email: caulier.gilles@free.fr\n\n"
                                  "Based on grab Pixie module implementation"),
                                  i18n("About Digikam screenshot plugin"));
}


/////////////////////////////////////////////////////////////////////////////////////////////

void ScreenGrabDialog::slotClose( void )
{
    // Write all settings in configuration file.

    m_config = new KConfig("digikamrc");
    m_config->setGroup("ScreenshotImages Settings");
    m_config->writeEntry("GrabDesktop", m_desktopCB->isChecked());
    m_config->writeEntry("HideDigikam", m_hideCB->isChecked());
    m_config->writeEntry("Delay", m_delay->value());
    m_config->sync();
    delete m_config;

    close();
    delete this;
}


/////////////////////////////////////////////////////////////////////////////////////////////

void ScreenGrabDialog::slotCheckIfDelay()
{
    // Hiding dialog first

    m_oldPos = pos();
    move(-4000, -4000);  // Move the windows instead an hide operation because the mouse
                         // cursor cannot be changed in this case.

    // Hiding the Digikam windows

    if (m_hideCB->isChecked())
       kapp->mainWidget()->hide();

    kapp->processEvents();
    QApplication::syncX();

    if (m_delay->value())
       {
       QTimer::singleShot(m_delay->value()*1000, this, SLOT(slotStartGrab()));
       kapp->processEvents();
       }
    else
       slotStartGrab();
}


/////////////////////////////////////////////////////////////////////////////////////////////

void ScreenGrabDialog::slotStartGrab()
{
    QPixmap pixmap;

    if (m_desktopCB->isChecked())
       {
       QApplication::syncX();
       sleep(1);
       pixmap = QPixmap::grabWindow(QApplication::desktop()->winId());
       }
    else
       {
       Display *disp = QApplication::desktop()->x11Display();
       Window rootWin, childWin;
       int rootX, rootY, winX, winY;
       unsigned int mask;

       m_winSelected = false;
       m_inSelect = true;
       grabMouse(crossCursor);

       QApplication::syncX();

       while(!m_winSelected)
           kapp->processEvents();

       releaseMouse();
       m_inSelect = false;
       XQueryPointer(disp, DefaultRootWindow(disp), &rootWin, &childWin,
                     &rootX, &rootY, &winX, &winY, &mask);

       QApplication::flushX();

       if (!childWin)
          {
          kdDebug() << "ScreenshotImage: no child window selected, using root" << endl;
          pixmap = QPixmap::grabWindow(QApplication::desktop()->winId());
          }
       else
          pixmap = getWindow(childWin, false);
       }

    KNotifyClient::beep();

    // Restore the Digikam windows

    if (m_hideCB->isChecked())
       {
       kapp->mainWidget()->show();
       QApplication::syncX();
       }

    if (pixmap.isNull())
       {
       KMessageBox::sorry(0, i18n("Unable to take snapshot!"),
                          i18n("Screenshot Error"));
       return;
       }

    m_screenshotImage = pixmap.convertToImage();

    m_acquireImageDialog = new AcquireImageDialog(0, m_screenshotImage);
    m_acquireImageDialog->setMinimumWidth(400);
    m_acquireImageDialog->exec();

    move(m_oldPos);
}


/////////////////////////////////////////////////////////////////////////////////////////////

void ScreenGrabDialog::mousePressEvent(QMouseEvent *ev)
{
    if (m_inSelect)
        m_winSelected = true;
}


/////////////////////////////////////////////////////////////////////////////////////////////

QPixmap ScreenGrabDialog::getWindow(Window window, bool nobdrs)
{
    XWindowAttributes win_info;
    int absx, absy, x, y;
    int width, height;
    int dwidth, dheight;
    int bw;
    Window dummywin;
    QPixmap pix;
    XImage *image;

    Display *dpy = QApplication::desktop()->x11Display();
    qWarning("Getting target window information");

    if (!XGetWindowAttributes(dpy, window, &win_info))
       {
       qWarning("Can't get target window attributes.");
       return(pix);
       }

    if (!XTranslateCoordinates (dpy, window, RootWindow(dpy, DefaultScreen(dpy)),
                                0, 0, &absx, &absy, &dummywin))
       {
       qWarning("Unable to translate window coordinates (%d,%d)", absx, absy);
       return(pix);
       }

    win_info.x = absx;
    win_info.y = absy;
    width = win_info.width;
    height = win_info.height;
    bw = 0;

    if (!nobdrs)
       {
       absx -= win_info.border_width;
       absy -= win_info.border_width;
       bw = win_info.border_width;
       width += (2 * bw);
       height += (2 * bw);
       }

    dwidth = QApplication::desktop()->width();
    dheight = QApplication::desktop()->height();

    // clip to window

    if (absx < 0)
        width += absx, absx = 0;

    if (absy < 0)
        height += absy, absy = 0;

    if (absx + width > dwidth)
        width = dwidth - absx;

    if (absy + height > dheight)
        height = dheight - absy;

    x = absx - win_info.x;
    y = absy - win_info.y;

    image = XGetImage(dpy, window, x, y, width, height, AllPlanes, ZPixmap);

    if (!image)
       {
       qWarning("Unable to get image at %dx%d+%d+%d", width, height, x, y);
       return(pix);
       }

    GC gc = qt_xget_temp_gc(qt_xscreen(), false);
    pix.resize(width, height);
    XPutImage(dpy, pix.handle(), gc, image, 0, 0, 0, 0, width, height);
    XFree(image);
    return(pix);
}
