//////////////////////////////////////////////////////////////////////////////
//
//    KIMG2MPG.H
//
//    Copyright (C) 2003 Gilles CAULIER <caulier.gilles@free.fr>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////


#ifndef KImg2mpgData_included
#define KImg2mpgData_included

// Qt includes

#include <qwidget.h>
#include <qbuttongroup.h>
#include <qcombobox.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qdialog.h>
#include <qprogressbar.h>
#include <qtimer.h>
#include <qdir.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qlistbox.h>
#include <qheader.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qprocess.h>
#include <qdatetime.h>
#include <qlayout.h>

// C++ includes

#include <iostream>

// KDElib includes

#include <kprogress.h>
#include <kprocess.h>
#include <klocale.h>
#include <kconfig.h>
#include <kdialog.h>
#include <kfiledialog.h>
#include <kcolordialog.h>
#include <khelpmenu.h>
#include <kpopupmenu.h>
#include <klineedit.h>
#include <kiconloader.h>
#include <kcolorbtn.h>
#include <klistbox.h>
#include <kbuttonbox.h>
#include <kurl.h>
#include <kimageio.h>
#include <kmessagebox.h>
#include <kapplication.h>
#include <kstandarddirs.h>
#include <kglobalsettings.h>

// Local includes

#include "ImageFilePreview.h"
#include "kshowdebuggingoutput.h"
#include "OptionsDialog.h"
#include "CheckBinProg.h"

#include <digikam/albummanager.h>
#include <digikam/albuminfo.h>

class KImg2mpgData : public KDialog
{
Q_OBJECT

public:
  KImg2mpgData(QWidget* parent = 0, const char * name = 0);
  virtual ~KImg2mpgData();
  void show();
  void ShowNumberImages( int Number );
  void closeEvent(QCloseEvent*);
  QPixmap LoadIcon( QString Name, int Group );
  void addItems(const QStringList& fileList);
  void writeSettings();
  void readSettings();

  OptionsDialog* OptionDlg;
  QPushButton*   Encodebutton;
  QGroupBox*     AudioInputFilename;

public slots:
  void reset();
  void readStderr(KProcess *proc, char *buffer, int buflen);
  void EncodeDone(KProcess* );
  void slotMPEGFilenameDialog( void );
  void slotAudioFilenameDialog( void );
  void slotImagesFilesButtonAdd( void );
  void slotImagesFilesButtonDelete( void );
  void slotImagesFilesButtonUp( void );
  void slotImagesFilesButtonDown( void );
  void slotEncode( void );
  void slotOptions( void );
  void slotClose( void );
  void slotAbout( void );
  void slotHelp( void );
  void slotImagesFilesSelected( QListBoxItem *item );
  void SlotPortfolioDurationChanged ( int );
  void slotOptionDlgOkClicked( void );

private:
  QString               m_VideoFormatConfig;
  QString               m_VideoTypeConfig;
  QString               m_ImageDurationConfig;
  QString               m_TransitionSpeedConfig;
  QColor                m_BackgroundColorConfig;
  QString               m_MPEGOutputFileConfig;
  QString               m_AudioInputFileConfig;
  QString               m_IMBinFolderConfig;
  QString               m_MJBinFolderConfig;
  QString               m_TmpFolderConfig;
  KConfig*              m_config;
  QTime                 m_EncodingDuration, m_DurationTime;

  QString               m_NoneLabel;
  bool                  m_Abort;
  bool                  m_Encoding;
  QString               m_EncodeString;
  QString               m_DebugOuputMessages;
  pid_t                 m_Img2mpgPidNum;
  KProcess*             m_Proc;
  QLabel*               m_ImageLabel;
  QLabel*               m_frame;
  QLabel*               m_label1;
  QLabel*               m_label3;
  QLabel*               m_label4;
  QLabel*               m_label5;
  QLabel*               m_label6;
  QLabel*               m_label7;
  KProgress*            m_progress;
  QPushButton*          m_quitbutton;
  QPushButton*          m_aboutbutton;
  QPushButton*          m_optionsbutton;
  QPushButton*          m_helpbutton;
  QComboBox*            m_VideoTypeComboBox;
  QComboBox*            m_VideoFormatComboBox;
  QComboBox*            m_TransitionComboBox;
  QSpinBox*             m_DurationImageSpinBox;
  QGroupBox*            m_MPEGOutputFilename;
  KLineEdit*            m_MPEGOutputEDITFilename;
  QPushButton*          m_MPEGOutputBUTTONFilename;
  KIconLoader*          m_Icons;
  KColorButton*         m_BackgroundColorButton;
  KLineEdit*            m_AudioInputEDITFilename;
  QPushButton*          m_AudioInputBUTTONFilename;
  QGroupBox*            m_ImagesFilesGroup;
  KListBox*             m_ImagesFilesListBox;
  KButtonBox*           m_ImagesFilesButtonBox;
  QPushButton*          m_ImagesFilesButtonAdd;
  QPushButton*          m_ImagesFilesButtonDelete;
  QPushButton*          m_ImagesFilesButtonUp;
  QPushButton*          m_ImagesFilesButtonDown;
  QString               m_CommandLine;

  class ImageFilePreview*     PreviewImage;
  class KShowDebuggingOutput* DebuggingDialog;

  void RemoveTmpFiles( void );
  bool DeleteDir(QString dirname);
  bool deldir(QString dirname);
};

#endif // KImg2mpgData_included
