//////////////////////////////////////////////////////////////////////////////
//
//    SENDIMAGESDIALOG.CPP
//
//    Copyright (C) 2003 Gilles CAULIER <caulier.gilles@free.fr>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////

#include <digikam/albummanager.h>
#include <digikam/albuminfo.h>

#include "SendImagesDialog.h"
#include "listImagesErrorDialog.h"


//////////////////////////////////// CONSTRUCTOR ////////////////////////////////////////////

SendImagesDialog::SendImagesDialog(QWidget *parent, QString TmpPath)
                 : KDialogBase( parent, "SendImagesDialog", true,
                 i18n("Send Mail Images options"), Help|Ok|Cancel|No, Ok, true, i18n("&About"))
{
    setHelp("plugin-sendimages.anchor", "digikam");
    QString whatsThis;
    m_mozillaTimer = new QTimer(this);
    m_tempPath = TmpPath;
    QWidget* box = new QWidget( this );
    setMainWidget(box);
    QVBoxLayout *dvlay = new QVBoxLayout( box, 10, spacingHint() );

    //---------------------------------------------

    QHBoxLayout *hlay10  = new QHBoxLayout( );
    dvlay->addLayout( hlay10 );

    QLabel *m_mailAgentLabel = new QLabel( i18n("Mail agent:"), box);

    m_mailAgentName = new QComboBox( false, box );
    m_mailAgentName->insertItem( "Balsa" );
    m_mailAgentName->insertItem( "Evolution" );
    m_mailAgentName->insertItem( "Kmail" );
    m_mailAgentName->insertItem( "Mozilla" );
    m_mailAgentName->insertItem( "Netscape" );
    m_mailAgentName->insertItem( "Sylpheed" );
    m_mailAgentName->insertItem( "Thunderbird" );
    m_mailAgentName->setCurrentText( "Kmail" );
    QWhatsThis::add( m_mailAgentName, i18n("<p>Select here your prefered external mail agent program."
                                           "These mail agents version are supported:<p>"
                                           "<b>Balsa</b>: >= 2.x<p>"
                                           "<b>Evolution</b>: >= 1.4<p>"
                                           "<b>Kmail</b>: >= 1.3<p>"
                                           "<b>Mozilla</b>: >= 1.4<p>"
                                           "<b>Netscape</b>: >= 7.x<p>"
                                           "<b>Sylpheed</b>: >= 0.9<p>"
                                           "<b>Thunderbird</b>: >= 0.4<p>") );

    hlay10->addWidget( m_mailAgentLabel );
    hlay10->addStretch( 1 );
    hlay10->addWidget( m_mailAgentName );

    //---------------------------------------------

    m_addComments = new QCheckBox( i18n("Add comments in attached file"), box);
    QWhatsThis::add( m_addComments, i18n("<p>If you enable this option, all Album and images comments "
                                         "will be added like an attached file.") );
    dvlay->addWidget( m_addComments );

    //---------------------------------------------

    m_changeImagesProp = new QCheckBox(i18n("Change images properties to send"), box);
    dvlay->addWidget(m_changeImagesProp);
    QWhatsThis::add( m_changeImagesProp, i18n("<p>If you enable this option, "
                     "all images to send can be resized and recompressed.") );
    m_changeImagesProp->setChecked( true );

    QHBoxLayout *hlay12  = new QHBoxLayout();
    dvlay->addLayout( hlay12 );

    m_imagesResize = new QComboBox(false, box);
    m_imagesResize->insertItem(i18n("very small"));
    m_imagesResize->insertItem(i18n("small"));
    m_imagesResize->insertItem(i18n("medium"));
    m_imagesResize->insertItem(i18n("big"));
    m_imagesResize->insertItem(i18n("very big"));
    m_imagesResize->setCurrentText (i18n("small"));
    whatsThis = i18n("<p>Select here the images size to send:<p>"
                     "<b>%1</b>: 320 pixels size<p>"
                     "<b>%2</b>: 640 pixels size<p>"
                     "<b>%3</b>: 800 pixels size<p>"
                     "<b>%4</b>: 1024 pixels size<p>"
                     "<b>%5</b>: 1280 pixels size<p>")
                     .arg(i18n("very small"))
                     .arg(i18n("small"))
                     .arg(i18n("medium"))
                     .arg(i18n("big"))
                     .arg(i18n("very big"));
    QWhatsThis::add( m_imagesResize, whatsThis );

    m_labelImageSize = new QLabel( i18n("New images size:"), box);
    hlay12->addWidget( m_labelImageSize );
    m_labelImageSize->setBuddy( m_imagesResize );
    hlay12->addStretch( 1 );
    hlay12->addWidget(m_imagesResize);

    //---------------------------------------------

    m_imageCompression = new KIntNumInput(75, box);
    m_imageCompression->setRange(1, 100, 1, true );
    m_imageCompression->setLabel( i18n("New images compression:") );
    dvlay->addWidget( m_imageCompression );
    whatsThis = i18n("<p>The new compression value of images to send:<p>");
    whatsThis = whatsThis + i18n("<b>1</b>: very high compression<p>"
                                 "<b>25</b>: high compression<p>"
                                 "<b>50</b>: medium compression<p>"
                                 "<b>75</b>: low compression (default value)<p>"
                                 "<b>100</b>: no compression");

    QWhatsThis::add( m_imageCompression, whatsThis);

    //---------------------------------------------

    QHBoxLayout *hlay13  = new QHBoxLayout( );
    dvlay->addLayout( hlay13 );

    m_imagesFormat = new QComboBox(false, box);
    m_imagesFormat->insertItem("JPEG");
    m_imagesFormat->insertItem("PNG");
    m_imagesFormat->setCurrentText ("JPEG");
    whatsThis = i18n("<p>Select here the images files format to send.<p>");
    whatsThis = whatsThis + i18n("<b>JPEG</b>: The Joint Photographic Experts Group file format is a good Web file format but it use a compression with data lost.<p>"
                     "<b>PNG</b>: the Portable Network Graphics format is an extensible file format for the lossless, portable, well-compressed storage of raster images. PNG provides a patent-free replacement for GIF and can also replace many common uses of TIFF. PNG is designed to work well in online viewing applications, such as the World Wide Web, so it is fully streamable with a progressive display option. Also, PNG can store gamma and chromaticity data for improved color matching on heterogeneous platforms.");
    QWhatsThis::add( m_imagesFormat, whatsThis );

    m_labelImageFormat = new QLabel( i18n("Images file format:"), box);
    hlay13->addWidget( m_labelImageFormat );
    m_labelImageFormat->setBuddy( m_imagesFormat );
    hlay13->addStretch( 1 );
    hlay13->addWidget(m_imagesFormat);

    connect(m_mozillaTimer, SIGNAL(timeout()), this, SLOT(slotMozillaTimeout()));

    connect(m_changeImagesProp, SIGNAL(toggled(bool)), m_labelImageSize, SLOT(setEnabled(bool)));
    connect(m_changeImagesProp, SIGNAL(toggled(bool)), m_imagesResize, SLOT(setEnabled(bool)));
    connect(m_changeImagesProp, SIGNAL(toggled(bool)), m_imageCompression, SLOT(setEnabled(bool)));
    connect(m_changeImagesProp, SIGNAL(toggled(bool)), m_labelImageFormat, SLOT(setEnabled(bool)));
    connect(m_changeImagesProp, SIGNAL(toggled(bool)), m_imagesFormat, SLOT(setEnabled(bool)));

    connect(this, SIGNAL(user1Clicked()), this, SLOT(slotAbout()));

    //---------------------------------------------

    // Read all settings from configuration file.

    m_config = new KConfig("digikamrc");
    m_config->setGroup("SendImages Settings");

    m_mailAgentName->setCurrentText(m_config->readEntry("MailAgentName", "Kmail"));

    if (m_config->readEntry("ImagesChangeProp", "true") == "true")
        m_changeImagesProp->setChecked( true );
    else
        m_changeImagesProp->setChecked( false );

    m_imagesResize->setCurrentItem(m_config->readNumEntry("ImageResize", 1));    // Small size used by default...
    m_imageCompression->setValue(m_config->readNumEntry("ImageCompression", 75));
    m_imagesFormat->setCurrentText(m_config->readEntry("ImageFormat", "JPEG"));

    if (m_config->readEntry("AddComments", "true") == "true")
        m_addComments->setChecked( true );
    else
        m_addComments->setChecked( false );

    delete m_config;
}


//////////////////////////////////// DESTRUCTOR /////////////////////////////////////////////

SendImagesDialog::~SendImagesDialog()
{
}


//////////////////////////////////////// SLOTS //////////////////////////////////////////////


void SendImagesDialog::slotAbout( void )
{
    KMessageBox::about(this, i18n("A Digikam plugin for send mail images\n\n"
                                  "Author: Gilles Caulier\n\n"
                                  "Email: caulier.gilles@free.fr\n"),
                                  i18n("About Digikam Send Mail Images"));
}

/////////////////////////////////////////////////////////////////////////////////////////////////////

void SendImagesDialog::slotCancelled()
{
    m_cancelled = true;
    removeTmpFiles();
}


/////////////////////////////////////////////////////////////////////////////////////////////

  void SendImagesDialog::slotOk()
{
    close();

    // Write all settings in configuration file.

    m_config = new KConfig("digikamrc");
    m_config->setGroup("SendImages Settings");
    m_config->writeEntry("MailAgentName", m_mailAgentName->currentText());
    m_config->writeEntry("AddComments", m_addComments->isChecked());
    m_config->writeEntry("ImagesChangeProp", m_changeImagesProp->isChecked());
    m_config->writeEntry("ImageResize", m_imagesResize->currentItem());
    m_config->writeEntry("ImageCompression", m_imageCompression->value());
    m_config->writeEntry("ImageFormat", m_imagesFormat->currentText());
    m_config->sync();
    delete m_config;

    // Prepare data to send.

    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();
    album->openDB();
    QString ImageCommentsText = "";
    QString AlbumCommentsText = "";
    QStringList imagesfileList = album->getSelectedItemsPath();
    QStringList imagesResizedWithError;
    m_cancelled = false;

    // Prepare Albums comments.

    if ( m_addComments->isChecked() == true )
       {
       QString commentAlbum;

       if ( album->getComments() != "" )
           commentAlbum = album->getComments();
       else
           commentAlbum = i18n("no comment");

       AlbumCommentsText = i18n("Images from Album \"%1\" [%2]:\n").arg(album->getTitle()).arg(commentAlbum);
       }

    m_progressDlg = new QProgressDialog(0, "progressDlg", true );
    QObject::connect(m_progressDlg, SIGNAL( cancelled() ), this, SLOT( slotCancelled() ) );
    m_progressDlg->setCaption( i18n("Creating images"));
    m_progressDlg->setCancelButtonText(i18n("&Cancel"));
    m_progressDlg->show();

    m_progressDlg->setTotalSteps( imagesfileList.count() );
    int imgIndex = 0;

    // Main loop for data send creation.

    for ( QStringList::Iterator it = imagesfileList.begin(); !m_cancelled && (it != imagesfileList.end()); ++it )
        {
        QString ItemName = *it;
        ItemName = ItemName.section( '/', -1 );

        m_progressDlg->setLabelText( i18n("Creating image \"%1\" to send").arg(ItemName) );
        m_progressDlg->setProgress( imgIndex );
        kapp->processEvents();
        imgIndex++;

        // Prepare resized target images to send.

        if ( m_changeImagesProp->isChecked() == true )      // Prepare resizing images and items comments.
           {
           QString imageFormat = m_imagesFormat->currentText();
           QString imageName = ItemName.left( ItemName.findRev('.',-1) );
           QString imageNameFormat = imageName + extension(imageFormat);
           int sizeFactor = getSize( m_imagesResize->currentText() );
           int imageCompression = m_imageCompression->value();
           qDebug("Resizing '%s' -> '%s%s' (%s)", ItemName.ascii(), m_tempPath.ascii(), imageNameFormat.ascii(), imageFormat.ascii());

           if ( ResizeImage( *it, m_tempPath, imageFormat, imageNameFormat, sizeFactor, imageCompression) == false )
               {         // Resized images failed...
               m_progressDlg->setLabelText( i18n("Creating image \"%1\" failed").arg(ItemName) );
               kapp->processEvents();
               imagesResizedWithError.append(*it);
               }
           else          // Resized images OK...
               {
               // List of files to send.

               m_imagesSendList.append (m_tempPath + imageNameFormat);

               // Prepare items comments.

               if ( m_addComments->isChecked() == true )
                  {
                  QString commentItem;

                  if ( album->getItemComments(ItemName) != "" )
                     commentItem = album->getItemComments(ItemName);
                  else
                     commentItem = i18n("no comment");

                  ImageCommentsText = ImageCommentsText + i18n("Comments for image \"%1\": %2\n").arg(imageNameFormat).arg(commentItem);
                  }
               }
           }
        else     // No resize images operations...
           {
           // List of files to send.

           m_imagesSendList.append (*it);

           // Prepare items comments.

           if ( m_addComments->isChecked() == true )
              {
              QString ItemName = *it;
              ItemName = ItemName.section( '/', -1 );
              QString commentItem;

              if ( album->getItemComments(ItemName) != "" )
                 commentItem = album->getItemComments(ItemName);
              else
                 commentItem = i18n("no comment");

              if ( album->getItemComments(ItemName) != "" )
                 ImageCommentsText = ImageCommentsText + i18n("Comments for image \"%1\": %2\n").arg(ItemName).arg(commentItem);
              }
           }
        }

    delete m_progressDlg;

    if ( m_cancelled == true ) // Resizing images process canceled by user -> bye without removed tmp folder (already done!)...
       {
       album->closeDB();
       delete this;
       return;
       }

    if ( imagesResizedWithError.isEmpty() == false )    // If we have some errors during resized images process...
       {
       listImagesErrorDialog *ErrorImagesDialog = new listImagesErrorDialog(0, i18n("Error during resize images process"), i18n("Cannot resize this images files :"), i18n("Do you want added this images files like attachments (not resizing)?"), imagesResizedWithError);
       int ValRet = ErrorImagesDialog->exec();

       switch (ValRet)
         {
         case KDialogBase::Yes :         // Added source image files instead resized images...

            for ( QStringList::Iterator it = imagesResizedWithError.begin(); it != imagesResizedWithError.end(); ++it )
                {
                m_imagesSendList.append (*it);
                kapp->processEvents();
                if ( m_addComments->isChecked() == true )
                   {
                   QString ItemName = *it;
                   ItemName = ItemName.section( '/', -1 );
                   QString commentItem;

                   if ( album->getItemComments(ItemName) != "" )
                      commentItem = album->getItemComments(ItemName);
                   else
                      commentItem = i18n("no comment");

                   ImageCommentsText = ImageCommentsText + i18n("Comments for image \"%1\": %2\n").arg(ItemName).arg(commentItem);
                   }
                }

            break;
         case KDialogBase::No :         // Do nothing...
            break;
         case KDialogBase::Cancel :     // Stop process...
            album->closeDB();
            removeTmpFiles();
            delete this;
            return;
            break;
         }
       }

    album->closeDB();

    // Create a text file with the comments.

    if ( m_addComments->isChecked() == true )
       {
       QFile commentsFile( m_tempPath + i18n("comments.txt") );
       QTextStream stream( &commentsFile );
       commentsFile.open( IO_ReadWrite );
       stream << AlbumCommentsText + "\n" + ImageCommentsText << "\n";
       commentsFile.close();
       m_imagesSendList.append( m_tempPath + i18n("comments.txt") );
       }

    // Invoke mailer agent.

    if ( m_mailAgentName->currentText() == "Kmail" )      // Kmail agent call.
       {
       KApplication::kApplication()->invokeMailer(
                       QString::null,                                  // Destination address.
                       QString::null,                                  // Carbon Copy address.
                       QString::null,                                  // Blind Carbon Copy address
                       QString::null,                                  // Message Subject.
                       QString::null,                                  // Message Body.
                       QString::null,                                  // Message Body File.
                       m_imagesSendList);                              // Images attachments (+ comments).
       }

    if ( m_mailAgentName->currentText() == "Sylpheed" )    // Sylpheed mail agent call.
       {
       m_mailAgentProc = new KProcess;
       *m_mailAgentProc << "sylpheed" << "--compose";
       *m_mailAgentProc << "--attach";

       for ( QStringList::Iterator it = m_imagesSendList.begin() ; it != m_imagesSendList.end() ; ++it )
           *m_mailAgentProc << *it;

       if (m_mailAgentProc->start() == false)
          KMessageBox::error(0, i18n("Cannot start '%1' program.\nPlease, check your installation!").arg(m_mailAgentName->currentText()));
       }

    if ( m_mailAgentName->currentText() == "Balsa" )    // Balsa mail agent call.
       {
       m_mailAgentProc = new KProcess;
       *m_mailAgentProc << "balsa" << "-m" << "mailto:";

       for ( QStringList::Iterator it = m_imagesSendList.begin() ; it != m_imagesSendList.end() ; ++it )
           {
           *m_mailAgentProc << "-a";
           *m_mailAgentProc << *it;
           }

       if (m_mailAgentProc->start() == false)
          KMessageBox::error(0, i18n("Cannot start '%1' program.\nPlease, check your installation!").arg(m_mailAgentName->currentText()));
       }

    if ( m_mailAgentName->currentText() == "Evolution" )    // Evolution mail agent call.
       {
       m_mailAgentProc = new KProcess;
       *m_mailAgentProc << "evolution";

       QString Temp = "mailto:?subject=";

       for ( QStringList::Iterator it = m_imagesSendList.begin() ; it != m_imagesSendList.end() ; ++it )
           {
           Temp.append("&attach=");
           Temp.append( *it );
           }

       *m_mailAgentProc << Temp;

       if (m_mailAgentProc->start() == false)
          KMessageBox::error(0, i18n("Cannot start '%1' program.\nPlease, check your installation!").arg(m_mailAgentName->currentText()));
       }

    // Mozilla | Netscape | Thunderbird mail agent call.

    if ( m_mailAgentName->currentText() == "Mozilla" || m_mailAgentName->currentText() == "Netscape" ||
         m_mailAgentName->currentText() == "Thunderbird" )
       {
       m_mailAgentProc = new KProcess;

       if (m_mailAgentName->currentText() == "Mozilla")
          *m_mailAgentProc << "mozilla" << "-remote";
       else
          if (m_mailAgentName->currentText() == "Thunderbird")
             *m_mailAgentProc << "thunderbird" << "-remote";
          else
             *m_mailAgentProc << "netscape" << "-remote";

       QString Temp = "xfeDoCommand(composeMessage,attachment='";

       for ( QStringList::Iterator it = m_imagesSendList.begin() ; it != m_imagesSendList.end() ; ++it )
           {
           Temp.append( "file://" );
           Temp.append( *it );
           Temp.append( "," );
           }

       Temp.append("')");

       *m_mailAgentProc << Temp;

       connect(m_mailAgentProc, SIGNAL(processExited(KProcess *)), this, SLOT(slotMozillaExited(KProcess*)));

       connect(m_mailAgentProc, SIGNAL(receivedStderr(KProcess *, char*, int)),
               this, SLOT(slotMozillaReadStderr(KProcess*, char*, int)));

       qDebug ("%s", Temp.ascii());

       if (m_mailAgentProc->start(KProcess::NotifyOnExit , KProcess::All) == false)
          KMessageBox::error(0, i18n("Cannot start '%1' program.\nPlease, check your installation!").arg(m_mailAgentName->currentText()));
       else return;
       }

    // Close this dialog.

    delete this;
}


////////////////////////////////////////// FONCTIONS ////////////////////////////////////////////////

int SendImagesDialog::getSize ( const QString &choice )
{
    if (choice == i18n("very small"))
        return (320);
    if (choice == i18n("small"))
        return (640);
    if (choice == i18n("medium"))
        return (800);
    if (choice == i18n("big"))
        return (1024);
    if (choice == i18n("very big"))
        return (1280);

    return (640); // Default value...
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void SendImagesDialog::slotMozillaExited(KProcess* proc)
{
    if ( m_mozillaStdErr.contains("No running window found.") == true )   // No remote Mozilla | Netscape |
       {                                                                  // Thunderbird env. loaded !
       m_mailAgentProc2 = new KProcess;                                   // Init a new env.

       if (m_mailAgentName->currentText() == "Mozilla")
          *m_mailAgentProc2 << "mozilla" << "-mail";
       else
          if (m_mailAgentName->currentText() == "Thunderbird")
             *m_mailAgentProc2 << "thunderbird" << "-mail";
          else
             *m_mailAgentProc2 << "netscape" << "-mail";

       if (m_mailAgentProc2->start() == false)     // Start an instance of mozilla mail agent before a remote call.
          {
          KMessageBox::error(0, i18n("Cannot start '%1' program.\nPlease, check your installation!").arg(m_mailAgentName->currentText()));
          delete this;
          }
       else
          {
          // Mozilla | Netscape | Thunderbird mail agent started correctly
          // -> start a remote mail agent with multiple attachments after the env. is loaded !

          m_mozillaTimer->start(5000, true);
          return; // Don't close this dialog -> will be closed by the end timer slot.
          }
       }
    else
       delete this;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void SendImagesDialog::slotMozillaTimeout(void)
{
    m_mailAgentProc3 = new KProcess;

    if (m_mailAgentName->currentText() == "Mozilla")
       *m_mailAgentProc3 << "mozilla" << "-remote";
    else
       if (m_mailAgentName->currentText() == "Thunderbird")
          *m_mailAgentProc3 << "thunderbird" << "-remote";
       else
          *m_mailAgentProc3 << "netscape" << "-remote";

    QString Temp = "xfeDoCommand(composeMessage,attachment='";

    for ( QStringList::Iterator it = m_imagesSendList.begin() ; it != m_imagesSendList.end() ; ++it )
        {
        Temp.append( "file://" );
        Temp.append( *it );
        Temp.append( "," );
        }

    Temp.append("')");

    *m_mailAgentProc3 << Temp;

    if (m_mailAgentProc3->start() == false)
       KMessageBox::error(0, i18n("Cannot start '%1' program.\nPlease, check your installation!").arg(m_mailAgentName->currentText()));

    // Close this dialog.

    delete this;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void SendImagesDialog::slotMozillaReadStderr(KProcess* proc, char *buffer, int buflen)
{
    m_mozillaStdErr = QString::fromLocal8Bit(buffer, buflen);
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

bool SendImagesDialog::ResizeImage( const QString &SourcePath, const QString &DestPath, const QString &ImageFormat,
                       const QString &ImageName, int SizeFactor, int ImageCompression)
{
QImage img;

if ( img.load(SourcePath) == true )
   {
   int w = img.width();
   int h = img.height();

   if( w > SizeFactor || h > SizeFactor )
       {
       if( w > h )
           {
           h = (int)( (double)( h * SizeFactor ) / w );

           if ( h == 0 ) h = 1;

           w = SizeFactor;
           Q_ASSERT( h <= SizeFactor );
           }
       else
           {
           w = (int)( (double)( w * SizeFactor ) / h );

           if ( w == 0 ) w = 1;

           h = SizeFactor;
           Q_ASSERT( w <= SizeFactor );
           }

       const QImage scaleImg(img.smoothScale( w, h ));

       if ( scaleImg.width() != w || scaleImg.height() != h )
           {
           qDebug("Resizing failed. Aborting.");
           return false;
           }

       img = scaleImg;
       }

    if ( !img.save(DestPath + ImageName, ImageFormat.latin1(), ImageCompression) )
       {
       qDebug("Saving failed with specific compression value. Aborting.");
       return false;
       }

    return true;
    }

return false;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

QString SendImagesDialog::extension(const QString& imageFileFormat)
{
    if (imageFileFormat == "PNG")
        return ".png";

    if (imageFileFormat == "JPEG")
        return ".jpg";

    Q_ASSERT(false);
    return "";
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void SendImagesDialog::removeTmpFiles(void)
{
    if (DeleteDir(m_tempPath) == false)
       KMessageBox::error(0, i18n("Cannot remove temporary folder %1!").arg(m_tempPath));
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

bool SendImagesDialog::DeleteDir(QString dirname)
{
if (dirname != "")
    {
    QDir dir;

    if (dir.exists ( dirname ) == true)
       {
       if (deldir(dirname) == false)
           return false;

       if (dir.rmdir( dirname ) == false )
           return false;
       }
    else
       return false;
    }
else
    return false;

return true;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

bool SendImagesDialog::deldir(QString dirname)
{
QDir *dir = new QDir(dirname);
dir->setFilter ( QDir::Dirs | QDir::Files | QDir::NoSymLinks );

const QFileInfoList* fileinfolist = dir->entryInfoList();
QFileInfoListIterator it(*fileinfolist);
QFileInfo* fi;

while( (fi = it.current() ) )
     {
     if(fi->fileName() == "." || fi->fileName() == ".." )
          {
          ++it;
          continue;
          }

     if( fi->isDir() )
          {
          if (deldir( fi->absFilePath() ) == false)
              return false;
          if (dir->rmdir( fi->absFilePath() ) == false)
              return false;
          }
     else
          if( fi->isFile() )
               if (dir->remove(fi->absFilePath() ) == false)
                   return false;

     kapp->processEvents();
     ++it;
     }

return true;
}

