/* ============================================================
 * File  : plugin_sendimages.cpp
 * Author: Gilles Caulier <caulier.gilles@free.fr>
 * Date  : 2003-11-04
 * Description : Digikam Send Mail Images Plugin.
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#include <unistd.h>

#include <qimage.h>

#include <klocale.h>
#include <kapplication.h>
#include <kglobal.h>
#include <kaction.h>
#include <kgenericfactory.h>
#include <klibloader.h>
#include <kconfig.h>
#include <kdebug.h>
#include <kiconloader.h>
#include <kinstance.h>
#include <kmessagebox.h>
#include <kstandarddirs.h>
#include <ktempfile.h>
#include <kscan.h>
#include <kimageio.h>

#include "plugin_sendimages.h"


K_EXPORT_COMPONENT_FACTORY( digikamplugin_sendimages,
                            KGenericFactory<Plugin_SendImages>("digikam"));

// -----------------------------------------------------------
Plugin_SendImages::Plugin_SendImages(QObject *parent, const char*, const QStringList&)
            : Digikam::Plugin(parent, "SendImages")
{
    setInstance(KGenericFactory<Plugin_SendImages>::instance());
    setXMLFile("plugins/digikamplugin_sendimages.rc");
    KGlobal::locale()->insertCatalogue("digikamplugin_sendimages");
    kdDebug() << "Plugin_SendImages plugin loaded" << endl;

    m_action_sendimages = new KAction (i18n("Email Images..."),      // Menu message.
                                        "mail_new",                  // Menu icon.
                                        CTRL+SHIFT+Key_M,            // default shortcut.
                                        this,
                                        SLOT(slotActivate()),
                                        actionCollection(),
                                        "send_images");


    m_action_sendimages->setEnabled(false);

    connect(Digikam::AlbumManager::instance(),
            SIGNAL(signalAlbumItemsSelected(bool)),
            SLOT(slotItemsSelected(bool)));
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

Plugin_SendImages::~Plugin_SendImages()
{
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void Plugin_SendImages::slotActivate()
{
    Digikam::AlbumInfo *album = Digikam::AlbumManager::instance()->currentAlbum();

    if (!album) return;

    QStringList imagesfileList = album->getSelectedItemsPath();

    if (imagesfileList.isEmpty() == true)
        {
        KMessageBox::sorry(0, i18n("Please select one or more photos to send."),
         i18n("Send Images"));
        return;
        }

    KStandardDirs dir;
    QString Tmp = dir.saveLocation("tmp", "digikam-sendimages-" + QString::number(getpid()) + "/");

    m_sendImagesDialog = new SendImagesDialog(0, Tmp);
    m_sendImagesDialog->show();
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

void Plugin_SendImages::slotItemsSelected(bool val)
{
    m_action_sendimages->setEnabled(val);
}

